﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;
using System.Collections.ObjectModel;
using System.Windows.Forms.Integration;
using System.Windows.Controls;

namespace FcTabControl
{
   public delegate bool CurrentTabPreCloseEvent(object sender, RoutedEventArgs e);

   public class FcTabItem : RadTabItem
    {

       #region Private Variables
        Dictionary<string, CurrentTabEvent> ContextItemList;

        public RoutedEvent PreCloseCheckEvent;
        private WindowsFormsHost FcUserCntlContainer;
        public event CurrentTabPreCloseEvent PreCloseEvent;

        public delegate void CurrentTabEvent(object sender, RoutedEventArgs e);
        public static readonly RoutedEvent TabClosingEvent = EventManager.RegisterRoutedEvent("TabClosing", RoutingStrategy.Bubble, typeof(RoutedEventHandler), typeof(FcTabItem));
       
       #endregion

        public event RoutedEventHandler TabClosing
        {
            add { AddHandler(TabClosingEvent, value); }
            remove { RemoveHandler(TabClosingEvent, value); }
        }

        public double HeaderWidth
        {
            get
            {
                if (this.Header!=null)
                {
                    return ((Grid)this.Header).Width;
                }
                return 0.0;
            }
            set
            {
                if (this.Header != null)
                {
                    ((Grid)this.Header).Width=value;
                }
            }
        }

        public double HeaderHeight
        {
            get
            {
                if (this.Header != null)
                {
                    return ((Grid)this.Header).Height;
                }
                return 0.0;
            }
            set
            {
                if (this.Header != null)
                {
                    ((Grid)this.Header).Height = value;
                }
            }
        }


        public FcTabItem(string stringTitle, System.Windows.Forms.UserControl fcUserControl, System.Drawing.Image menuImage)
        {
            AddHeaderStyle(stringTitle,menuImage);
            AddContentTemplate(fcUserControl);
            
            ContextItemList = new Dictionary<string, CurrentTabEvent>();

            ContextItemList.Add("Close",CloseCurrentTab);
            ContextItemList.Add("Close All But This", CloseAllButCurrentTab);
        }

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            ContextMenu TabcontextMenu = new ContextMenu();
            TabcontextMenu.Items.Clear();
            TabcontextMenu.ItemsSource = GetAllTabContextMenuItem();
            this.ContextMenu = TabcontextMenu;
        }

        private void CloseCurrentTab(object sender, RoutedEventArgs e)
        {
            ((Telerik.Windows.Controls.RadTabControl)(this.Parent)).SelectedItem = this;
            if (this.PreCloseEvent != null)
            {
                if (!(this.PreCloseEvent(sender, e)))
                {
                    return;
                }
            }
            this.OnRaiseTabClosingEvent(new RoutedEventArgs(TabClosingEvent, this));
        }

        public void AddContextMenuItem(string MenuCaption, CurrentTabEvent AssignCurrentTabEvent)
        {
            ContextItemList.Add(MenuCaption, AssignCurrentTabEvent);
        }

        private void CloseAllButCurrentTab(object sender, RoutedEventArgs e)
        {
            bool IsClosedAll = false;
            int intTabItemIndex = ((Telerik.Windows.Controls.RadTabControl)(this.Parent)).Items.Count - 1;

            while ((!IsClosedAll) && (((Telerik.Windows.Controls.RadTabControl)(this.Parent)).Items.Count > 1))
            {
                FcTabItem fcItem;
                fcItem = (FcTabItem)((Telerik.Windows.Controls.RadTabControl)(this.Parent)).Items[intTabItemIndex];

                if (fcItem != this)
                {
                    if (fcItem.PreCloseEvent != null)
                    {
                        ((Telerik.Windows.Controls.RadTabControl)(this.Parent)).SelectedItem = fcItem;
                        if (!(fcItem.PreCloseEvent(sender, e)))
                        {
                            intTabItemIndex--;
                        }
                    }
                }

                intTabItemIndex--;
                ((Telerik.Windows.Controls.RadTabControl)(this.Parent)).Items.RemoveAt(intTabItemIndex);

                if (intTabItemIndex < 0)
                {IsClosedAll = true;
                } 
            }
        }

        void AddHeaderStyle(string Title, System.Drawing.Image menuImage)
        {

            Grid TabHeaderLayout = new Grid();
            TabHeaderLayout.MinWidth = 20;
            TabHeaderLayout.DataContext = Title;

            TabHeaderLayout.ColumnDefinitions.Add(new ColumnDefinition() { Width = new GridLength(0, GridUnitType.Star) });
            TabHeaderLayout.ColumnDefinitions.Add(new ColumnDefinition() { Width = new GridLength(0, GridUnitType.Auto) });
            TabHeaderLayout.ColumnDefinitions.Add(new ColumnDefinition() { Width = new GridLength(0, GridUnitType.Star) });


 

         //Title
            TextBlock textBlocklbl = new TextBlock() { Text = Title };
            

            //Close (Button Created here)
            RadButton rdButton = new RadButton();
            rdButton.Name = "BtnClose";
            rdButton.Width = 16;
            rdButton.Height = 16;
            rdButton.Margin = new Thickness(8, 1, -1, 1);
            rdButton.VerticalAlignment = VerticalAlignment.Center;


            //Button Context [x] created in the style
            rdButton.Style = ResourceDictionaryAccess.GetStyle("CloseButton");
            rdButton.Click += new RoutedEventHandler(closeTabItemButton_Click);

            //convert System.Drawing.Image to WPF image
            System.Drawing.Bitmap bmp = new System.Drawing.Bitmap(menuImage);
            IntPtr hBitmap = bmp.GetHbitmap();
            System.Windows.Media.ImageSource WpfBitmap = System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(hBitmap, IntPtr.Zero, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());

            Image imgBox = new Image();
            imgBox.Width = 16;
            imgBox.Height = 16;
            imgBox.Margin = new Thickness(1, 1, 5, 1);

            imgBox.Source = WpfBitmap;

            imgBox.SetValue(Grid.ColumnProperty, 0);
            textBlocklbl.SetValue(Grid.ColumnProperty, 1);
            rdButton.SetValue(Grid.ColumnProperty, 2);

            TabHeaderLayout.Children.Add(imgBox);
            TabHeaderLayout.Children.Add(textBlocklbl);
            TabHeaderLayout.Children.Add(rdButton);


            StackPanel DropDownContextLayout = new StackPanel() { Orientation = Orientation.Horizontal};
            DropDownContextLayout.DataContext = Title;

            Image imgBox2 = new Image();
            imgBox2.Width = 16;
            imgBox2.Height = 16;
            imgBox2.Margin = new Thickness(1, 1, 5, 1);
            imgBox2.Source = WpfBitmap;

            TextBlock textBlocklbl2 = new TextBlock() { Text = Title };
 
           // DropDownContextLayout.ColumnDefinitions.Add(new ColumnDefinition() { Width = new GridLength(5) });
           // DropDownContextLayout.ColumnDefinitions.Add(new ColumnDefinition() { Width = new GridLength(5) });

           // imgBox2.SetValue(Grid.ColumnProperty, 0);
           // textBlocklbl2.SetValue(Grid.ColumnProperty, 1);

            DropDownContextLayout.Children.Add(imgBox2);
            DropDownContextLayout.Children.Add(textBlocklbl2);

            this.DropDownContent = DropDownContextLayout;
            
            this.Header = TabHeaderLayout;
        }

        void AddContentTemplate(System.Windows.Forms.UserControl fcUserControl)
        {
            StackPanel Sk = new StackPanel();
            TextBox txt = new TextBox();
            ScrollViewer sv = new ScrollViewer();


            txt.Text = "Helloooooo";
            Sk.Children.Add(txt);
            FcUserCntlContainer = new WindowsFormsHost();

            FcUserCntlContainer.VerticalAlignment = VerticalAlignment.Stretch;
            FcUserCntlContainer.HorizontalAlignment = HorizontalAlignment.Left;
            FcUserCntlContainer.Child = fcUserControl;
            Sk.Children.Add(FcUserCntlContainer);

            sv.HorizontalScrollBarVisibility = ScrollBarVisibility.Auto;
            sv.VerticalScrollBarVisibility = ScrollBarVisibility.Auto;

            sv.Content = Sk;
            this.Content = sv;

        }

        void closeTabItemButton_Click(object sender, RoutedEventArgs e)
        {
            this.OnRaiseTabClosingEvent(new RoutedEventArgs(TabClosingEvent, this));
        }

        internal virtual void OnRaiseTabClosingEvent(RoutedEventArgs e)
        {
            this.RaiseEvent(e);
        }

        internal virtual List<MenuItem> GetAllTabContextMenuItem()
        {
            List<MenuItem> listContextMenuItems = new List<MenuItem>();
            try
            {
                foreach (KeyValuePair<String, CurrentTabEvent> ContextMenuListItem in ContextItemList)
                {
                    MenuItem cntxtMenuItem = new MenuItem();
                    cntxtMenuItem.Header = (string)ContextMenuListItem.Key;
                    if (ContextMenuListItem.Value != null)
                    {
                        cntxtMenuItem.Click += new RoutedEventHandler(ContextMenuListItem.Value);
                    }
                    listContextMenuItems.Add(cntxtMenuItem);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            return listContextMenuItems;
        }
     }
}
